''*********************************
''* CONNECT4_42_1.2.spin   1/2020 *
''* Author: Werner L. Schneider   *
''*********************************
''
'' Changes in v1.2
''
'' NO EEPROM Version added.
''
'' Changes in v1.1
''
'' Add F12 for Reboot
''
''
CON
    _clkmode    =  xtal1 + pll16x
    _xinfreq    =  5_000_000

    video       = 16                            ' P16-P23 VGA

    kbd         = 26                            ' P26 Keyboard Data
    kbc         = 27                            ' P27 Keyboard Clock

    scl         = 28                            ' P28 SCL Propeller EEPROM
    sda         = 29                            ' P29 SDA       "

  
OBJ

    i2c:    "I2CSPIN"                           ' Eeprom-Access                 Main Cog

    kb:     "KEYBOARD_EN"                       ' Keyboard Driver               need 1 Cog

    sound:  "PLAY"                              ' AY 3-8910 left/right Channel  need 3 Cog's

    time:   "TIME"                              ' Time Driver                   need 1 Cog

    vga:    "VGACOLOUR_640x480_8x16_FG16-BG16"  ' VGA Driver                    need 2 Cog's


VAR

    byte vgabuff[4800]                          ' VGA Text-Buffer 80x30 2400x2 Bytes 1. Byte Character 0..255, 2. Byte fgc + (bgc * 16)
    byte cursor[6]                              ' Cursor info array 
    long sync                                   ' sync used by VGA routine

    long pos                                    ' Global Screen-Pointer

    long xpos

    byte play1
    byte play2
    byte mirror
    byte animation

    byte actplayer

    byte num[4]

    byte rest[3]
    
    byte discs[7]

    byte play[42]

    byte save[42]


PUB main | key, x    
    init

    intro

    actplayer := 1                         
    rest[1] := 21 
    rest[2] := 21 

    printStart
    printPlay
    etime(0, play1, 0) 
    etime(75, play2, 0) 

    waitcnt((clkfreq / 1000) * 1000 + cnt)                   ' wait 1 s

    time.start

    repeat

        initMove

        if actplayer == 1
            time.set1
        else
            time.set2

        repeat

            key := kb.key
            if key <> 0

                if key == $52 or key == $72                  ' R or r
                    time.end
                    x := ((xpos // 7) * 7)+ 16
                    printDisc2(x, 1, $00)
                    rest[actplayer]++
                    printRest
                    dropDiscs
                    etime(0, play1, 0) 
                    etime(75, play2, 0) 
                    actplayer := 1
                    time.start
                    quit

                elseif key == $20 or key == $B3              ' Space or Arrow Down
                    if drop
                        if testLines                         ' 4 connected ?
                            time.end
                            dropDiscs
                            actplayer := 2
                            etime(0, play1, 0) 
                            etime(75, play2, 0) 
                            fill(32, 0, $0F, $00, 16, $20)
                            time.start
                            
                        if actplayer == 1
                            actplayer := 2
                        else
                            actplayer := 1

                        if mirror == 0
                            etime(0, play1, time.get1) 
                            etime(75, play2, time.get2) 
                        else
                            if actplayer == 2
                                etime(0, play2, time.get2) 
                                etime(75, play1, time.get1) 
                            else
                                etime(0, play1, time.get1) 
                                etime(75, play2, time.get2) 
                            automirror
                        
                        quit

                elseif key == $B0                            ' Arrow Left
                    move(0)

                elseif key == $B1                            ' Arrow Right
                    move(1)

                elseif key == $C1                            ' F2 Toggle Auto-Mirror
                    mirror ^= 1
                    strxy(32, 0, $0F, $00, @strF2+7)
                    if mirror == 0
                        strxy(44, 0, $0F, $00, @strOff)
                    else
                        strxy(44, 0, $0F, $00, @strOn)
                    if actplayer == 2
                        automirror
                        printRest

                    printTime
                    waitcnt((clkfreq / 1000) * 1000 + cnt)   ' wait 1 s
                    fill(32, 0, $0F, $00, 15, $20)

                elseif key == $C2                            ' F3 Toggle Animation
                    animation ^= 1
                    strxy(33, 0, $0F, $00, @strF3+7)
                    if animation == 0
                        strxy(43, 0, $0F, $00, @strOff)
                    else
                        strxy(43, 0, $0F, $00, @strOn)
                    waitcnt((clkfreq / 1000) * 1000 + cnt)   ' wait 1 s
                    fill(33, 0, $0F, $00, 13, $20)

                elseif key == $CB                            ' F12 Reboot
                    reboot

            printTime


PRI init | addr, magic

    i2c.start(scl, sda)

    i2c.read(1, $7FE0, @magic)     

    if magic == 76                 
        i2c.read(1, $7FE1, @play1) 
        i2c.read(1, $7FE2, @play2) 
        i2c.read(1, $7FE3, @mirror) 
        i2c.read(1, $7FE4, @animation) 

    else
        magic := 76
        play1 := $09
        play2 := $0B
        mirror := 0
        animation := 1
        saveConfig

    kb.start(kbd, kbc)                                    ' Start Keyboard Driver

    printIntro

    sound.start(10,11)                                    ' Start right Channel, left Channel

    vga.start(video, @vgabuff, @cursor, @sync)            ' Start Video Driver
    waitcnt(clkfreq * 1 + cnt)                            ' wait 1 second for cogs to start

    cursor[0] := 0
    cursor[1] := 0
    cursor[2] := 0
    cursor[3] := 0
    cursor[4] := 0
    cursor[5] := 0

    bytefill(@play, $00, 42)


PRI saveConfig | magic

    magic := 76

    i2c.write(1, $7FE0, @magic) 
    i2c.write(1, $7FE1, @play1) 
    i2c.write(1, $7FE2, @play2) 
    i2c.write(1, $7FE3, @mirror) 
    i2c.write(1, $7FE4, @animation) 


DAT
''********************************************************************
''* Functions for Gameplay                                           *
''********************************************************************
''
PRI dropDiscs | i, j, k, x, s, p

    bytefill(@save, $00, 42)

    fill(16, 23, $0C, $00, 48, $8F)

    repeat i from 29 to 24

        repeat j from 0 to 6
            discs[j] := play[j+35]

        repeat k from 34 to 0
            play[k+7] := play[k]
        repeat k from 0 to 6
            play[k] := 0
        
        printPlay

        repeat j from 0 to 6

            p := ((i-24) * 7) + j

            x := ((j // 7) * 7)+ 16

            s := ((||(cnt?)) // 3)
            if s == 1
                x++
            elseif s == 2
                x--

            save[p] := discs[j] 

            if save[p] == 1

                fill2(x, i, play1, $00, 6, $86)
                click(j)

            elseif save[p] == 2

                fill2(x, i, play2, $00, 6, $86)
                click(j) 

    waitcnt((clkfreq / 1000) * 2000 + cnt)

    repeat i from 0 to 5

        if i < 4
            printLeg1(i+24)
        elseif i == 4
            printLeg2
        elseif i == 5
            fill(13, 29, $00, $00, 54, $20)    

        repeat j from 0 to 6

            p := ((i) * 7) + j

            if save[p] == 1
                rest[1]++
                printRest
                sound.ch1B(@tone3)
                waitcnt((clkfreq / 1000) * 10 + cnt)
            elseif save[p] == 2
                rest[2]++
                printRest
                sound.ch1B(@tone3)
                waitcnt((clkfreq / 1000) * 10 + cnt)
            
        waitcnt((clkfreq / 1000) * 100 + cnt)      


PRI click(j) | s

    s := ((||(cnt?)) // 8) * 8

    if (j & 1) == 0
        sound.ch1C(@noise+s)            
    else
        sound.ch2C(@noise+s)            

    s := (((||(cnt?)) // 8) * 10) + 10

    waitcnt((clkfreq / 1000) * s + cnt)      


PRI testLines | i, o, win, key

    win := 0
    repeat i from 0 to 68
        o := i * 4
        if play[hTab[o]] == 1 and play[hTab[o+1]] == 1 and play[hTab[o+2]] == 1 and play[hTab[o+3]] == 1
            win := 1
            quit
            
    if win == 0
        repeat i from 0 to 68
            o := i * 4
            if play[hTab[o]] == 2 and play[hTab[o+1]] == 2 and play[hTab[o+2]] == 2 and play[hTab[o+3]] == 2
                win := 2
                quit

    if win <> 0

        printRestart

        strxy(32, 0, $0F, $00, @strTime)
        etime(43, $0F, time.get3)

        repeat

            sound.ch1B(@tone5)
            sound.ch2B(@tone5)

            play[htab[o]] := win 
            play[htab[o+1]] := win 
            play[htab[o+2]] := win 
            play[htab[o+3]] := win 
            printPlay

            repeat 500
                key := kb.key
                if key <> 0
                    if key == $52 or key == $72                    ' R or r
                        return -1
                waitcnt((clkfreq / 1000) * 1 + cnt)                ' wait 1 ms

            play[htab[o]] := 0 
            play[htab[o+1]] := 0 
            play[htab[o+2]] := 0 
            play[htab[o+3]] := 0 
            printPlay

            waitcnt((clkfreq / 1000) * 500 + cnt)                  ' wait 500 ms

    else

        if rest[1] == 0 and rest[2] == 0

            printRestart

            strxy(32, 0, $0F, $00, @strTime)
            etime(43, $0F, time.get3)

            bytemove(@save, @vgabuff+2152, 16)
            bytemove(@save+16, @vgabuff+2312, 16)

            repeat

                sound.ch1B(@tone6)
                sound.ch2B(@tone6)

                fill2(36, 13, $00, $0F, 8, $E0)                    ' Draw !
                fill2(36, 14, $00, $0F, 8, $F0)

                repeat 500
                    key := kb.key
                    if key <> 0
                        if key == $52 or key == $72                ' R or r
                            bytemove(@vgabuff+2152, @save, 16)
                            bytemove(@vgabuff+2312, @save+16, 16)
                            return -1

                    waitcnt((clkfreq / 1000) * 1 + cnt)            ' wait 1 ms

                bytemove(@vgabuff+2152, @save, 16)
                bytemove(@vgabuff+2312, @save+16, 16)

                waitcnt((clkfreq / 1000) * 500 + cnt)              ' wait 500 ms

    return 0


PRI drop | i, j, c, x, y, a, s, newpos

    if play[xpos] <> 0
        return 0

    time.end

    if actplayer == 1
        c := play1
    else
        c := play2

    x := ((xpos // 7) * 7)+ 16
    printDisc2(x, 1, $00)

    s := xpos 
    a := 1
    repeat 5
         if play[s+7] <> 0
             quit
         a++
         s += 7

    if actplayer == 1
        sound.ch1B(@tone2)
    else
        sound.ch2B(@tone2)

    y := 5
    newpos := xpos - 7

    repeat i from 1 to a
        printDisc1(x, y, c)
        waitcnt((clkfreq / 1000) * 50 + cnt)          ' wait 50 ms
        if i <> a
            printDisc1(x, y, $00)
        y += 3
        newpos += 7

    play[newpos] := actplayer 

    return -1


PRI automirror | i, j, s, d

    repeat i from 0 to 5
        s := i * 7
        d := s + 6
        repeat j from s to s+6
            save[d--] := play[j]
    repeat i from 0 to 41
        play[i] := save[i]
    printPlay


PRI move(s) | c, x

    if actplayer == 1
        c := play1
    else
        c := play2

    x := ((xpos // 7) * 7)+ 16
    printDisc2(x, 1, $00)


    if s == 0
        xpos--
        if xpos < 0
            xpos := 6
    else
        xpos++
        if xpos == 7
            xpos := 0

    x := ((xpos // 7) * 7)+ 16
    printDisc2(x, 1, c)

    if actplayer == 1
        sound.ch1B(@tone7)
    else
        sound.ch2B(@tone7)


PRI initMove | y, x, c, yy, xx, s

    if rest[1] == 0 and rest[2] == 0
        return
 
    if mirror == 0
        if actplayer == 1
            x := 2
            c := play1
        else
            x := 72
            c := play2

    else
        if actplayer == 1
            x := 2
            c := play1
        else
            x := 2
            c := play2

    sound.ch1B(@tone4)

    if animation == 1
        y := (((21 - rest[actplayer]) / 3) + 1) * 3 + 1 
        rest[actplayer]--
        printRest
        repeat yy from y to 1 step 3
            printDisc2(x, yy, c)
            waitcnt((clkfreq / 1000) * 100 + cnt)          ' wait 100 ms
            printDisc2(x, yy, $00)

        if mirror == 0
            if actplayer == 1
                s := 2
            else
                s := 72
        else
            s := 2

        repeat xx from s to 37 step 7
            printDisc2(xx, 1, c)
            waitcnt((clkfreq / 1000) * 100 + cnt)          ' wait 100 ms
            if xx <> 37
                printDisc2(xx, 1, $00)

    else
        rest[actplayer]--
        printRest
        printDisc2(37, 1, c)

    kb.clearkeys
    xpos := 3


PRI printRestart

    printxy(28, 23, $0C, $00, $96)
    strxy(29, 23, $0F, $0C, @strRestart)
    printxy(51, 23, $0C, $00, $97)


PRI printStart | i

    printxy(15, 4, $0C, $00, $8C)
    fill(16, 4, $0C, $00, 48, $8D)
    printxy(64, 4, $0C, $00, $8E)
    repeat i from 5 to 22
        fill(15, i, $0C, $00, 50, $9F) 
    printxy(14, 19, $0C, $00, $9C)
    printxy(65, 19, $0C, $00, $9E)
    fill(16, 23, $0C, $00, 48, $8F)

    repeat i from 20 to 27
        strxy(14, i, $0C, $00, @strBar)
        strxy(64, i, $0C, $00, @strBar)

    printLeg2

    printRest


PRI printLeg1(y)

    strxy(14, y, $0C, $00, @strBar)
    fill(16, y, $00, $00, 48, $20)
    strxy(64, y, $0C, $00, @strBar)


PRI printLeg2

    strxy(13, 28, $0C, $00, @strLeg)
    fill(17, 28, $00, $00, 46, $20)
    strxy(63, 28, $0C, $00, @strLeg)


PRI printRest | x, y, a, r, act, nact, ac, nac

    if mirror == 1
        if actplayer == 1
            act := 1
            nact := 2
            ac := play1
            nac := play2
        else
            act := 2
            nact := 1
            ac := play2
            nac := play1
    else
        act := 1
        nact := 2
        ac := play1
        nac := play2

    x := 2
    y := 8
    a := rest[1] 
    r := 21 - a 
    if r <> 0
        repeat r
            fill2(x, y++, $00, $00, 6, $86)

    if a <> 0
        repeat a
            fill2(x, y++, ac, $00, 6, $86)

    x := 72
    y := 8
    a := rest[2] 
    r := 21 - a 
    if r <> 0
        repeat r
            fill2(x, y++, $00, $00, 6, $86)

    if a <> 0
        repeat a
            fill2(x, y++, nac, $00, 6, $86)


PRI printPlay | i, x, y

    repeat i from 0 to 41
        x := ((i // 7) * 7)+ 16  
        y := ((i / 7) * 3) + 5 
        if play[i] == 0
            printDisc1(x, y, $00)

        elseif play[i] == 1
            printDisc1(x, y, play1)

        elseif play[i] == 2
            printDisc1(x, y, play2)


PRI printDisc1(x, y, c)

    fill2(x, y, c, $0C, 6, $80)
    fill2(x, y+1, c, $0C, 6, $90)
    fill2(x, y+2, c, $0C, 6, $A0)


PRI printDisc2(x, y, c)

    fill2(x, y, c, $00, 6, $B0)
    fill2(x, y+1, c, $00, 6, $C0)
    fill2(x, y+2, c, $00, 6, $D0)


PRI etime(x, c, t) | m, s

    if t > 3599
        t := 0

    m := t / 60
    s := t // 60

    decr(x+1, 0, c, $00, m)
    printxy(x+2, 0, c, $00, $3A)
    decr(x+4, 0, c, $00, s)


DAT
''********************************************************************
''* Functions for Intro-Screen                                       *
''********************************************************************
''
PRI intro | i, key, addr, q, c, n, keyerr, magic

    i := 0
    q := 0
    n := 0
    c := $0C

    keyerr := 1
    if kb.present
        keyerr := 0

    repeat
        repeat 1000
            key := kb.key
            if key <> 0
                if key == $53 or key == $73                ' S or s
                    q := 1
                    quit

                elseif key == $C0                          ' F1
                    if play1 == $09
                        play1 := $0B
                        play2 := $09
                    elseif play1 == $0B
                        play1 := $09
                        play2 := $0B
                    printConfig

                elseif key == $C1                          ' F2
                    mirror ^= 1
                    printConfig

                elseif key == $C2                          ' F3
                    animation ^= 1
                    printConfig

                elseif key == $CB                          ' F12 Reboot
                    reboot
                    
            waitcnt((clkfreq / 1000) * 1 + cnt)            ' wait 1 ms

        introanim

        n++
        if n == 5
            n := 0
            c++
            if c == 16
                c := 8
            printHat(c)    

        i ^= 1
        if i == 1
            if keyerr == 1
                strxy(29, 29, $0B, $01, @strNoKey)
            sound.ch1A(@tone0)
        else
            if keyerr == 1
                fill(29, 29, $0F, $0C, 22, $20)
            sound.ch2A(@tone0)

        if q == 1
            quit

    cls($00, $00)                                      ' Clear Screen fg = $00 black, bg = $00 black   fg = 0..15, bg = 0..15

    saveConfig


PRI introanim | i

    i := 0
    pos := 160
    repeat 301
        letter(i++)
    pos := 1670
    repeat 7
        repeat 23
            letter(i++)
        pos += 74


PRI letter(i) | s

    if introdat[i] == 0
        print($0C, $00, $9F)

    elseif introdat[i] == 1
        print($00, $0C, $98)
        print($00, $0C, $99)

    elseif introdat[i] == 2
        s := ((||(cnt?)) // 2)
        if s == 0
            print($09, $0C, $98)
            print($09, $0C, $99)
        else
            print($0B, $0C, $98)
            print($0B, $0C, $99)


PRI printIntro | i, s

    i := 0
    pos := 0
    repeat 29
        repeat 6
            print($00, $0C, $98)
            print($00, $0C, $99)
        print($0C, $00, $9F)
        repeat 5
            repeat 5
                print($00, $0C, $98)
                print($00, $0C, $99)
            print($0C, $00, $9F)
        repeat 6
            print($00, $0C, $98)
            print($00, $0C, $99)

    introanim

    strxy(0, 29, $0F, $0C, @introtext)

    fill2(46, 19, $0C, $0F, 10, $A6)
    fill2(46, 20, $0C, $0F, 10, $B6)
    fill2(46, 21, $0C, $0F, 10, $C6)
    fill2(46, 22, $0C, $0F, 10, $D6)

    drawBox(4, 12, 26, 15, $0F, $00)

    strxy(15, 13, $0F, $00, @strHelp)
    strxy(6, 15, $0F, $00, @strPlay1)
    strxy(6, 16, $0F, $00, @strPlay2)
    strxy(6, 17, $0F, $00, @strF2)
    strxy(6, 18, $0F, $00, @strF3)
    strxy(6, 19, $0F, $00, @strMove)
    strxy(6, 20, $0F, $00, @strDrop1)
    strxy(6, 21, $0F, $00, @strDrop2)
    strxy(6, 22, $0F, $00, @strRe)
    strxy(6, 23, $0F, $00, @strF12)

    strxy(8, 25, $0F, $00, @strStart)

    drawBox(63, 21, 13, 6, $0F, $00)

    printHat($0C)

    printConfig


PRI printConfig

    printxy(22, 15, play1, $00, $98) 
    print(play1, $00, $99) 
    printxy(22, 16, play2, $00, $98) 
    print(play2, $00, $99) 
    if mirror == 0
        strxy(25, 17, $0F, $00, @strOff)
    else
        strxy(25, 17, $0F, $00, @strOn)
    if animation == 0
        strxy(25, 18, $0F, $00, @strOff)
    else
        strxy(25, 18, $0F, $00, @strOn)

    sound.ch1B(@tone1)
    sound.ch2B(@tone1)
    saveConfig


PRI printHat(c)

    strxy(65, 22, c, $00, @strHat1)
    strxy(65, 23, c, $00, @strHat2)
    strxy(65, 24, c, $00, @strHat3)
    strxy(65, 25, c, $00, @strHat4)


DAT
''********************************************************************
''* Miscellaneous Functions                                          *
''********************************************************************
''
PRI printTime

    if mirror == 0
        etime(0, play1, time.get1) 
        etime(75, play2, time.get2) 
    else
        if actplayer == 2
            etime(0, play2, time.get2) 
            etime(75, play1, time.get1) 
        else
            etime(0, play1, time.get1) 
            etime(75, play2, time.get2) 


PRI cls(fgc, bgc)

    wordfill(@vgabuff, (fgc + (bgc *16)) * 256 + $20 , 2400)


PRI getPos(x, y)

    pos := (x + y * 80) * 2 

PRI fill(x, y, fgc, bgc, c, v)

    getPos(x, y)
    repeat c
        print(fgc, bgc, v)


PRI fill2(x, y, fgc, bgc, c, v)

    getPos(x, y)
    repeat c
        print(fgc, bgc, v++)


PRI fill3(fgc, bgc, char, len) | x
''************************************************
''* Fill len of Char's with char                 *
''* Foreground.Color fgc                         *
''* Background-Color bgc                         *
''* at current pos (x + y * 80) * 2              * 
''************************************************

    repeat x from 1 to len
        print(fgc, bgc, char)


PRI drawBox(x, y, w, h, fgc, bgc) | t, m, b, i
''************************************************
''* Draw Box at x, y with w = width, h = height  *
''* Foreground.Color fgc                         *
''* Background-Color bgc                         *
''************************************************

    t := (y * 80 + x) * 2
    m := t + 160
    b := m + ((h-2) * 160)
    pos := t  
    print(fgc, bgc, $7B)
    fill3(fgc, bgc, $5B, w-2)
    print(fgc, bgc, $7C)
    repeat i from 0 to h-3
        pos := m + i * 160
        print(fgc, bgc, $5C)
        fill3($0F, bgc, $20, w-2)
        print(fgc, bgc, $5C)   
    pos := b   
    print( fgc, bgc, $7D)
    fill3(fgc, bgc, $5B, w-2)
    print(fgc, bgc, $7E)


PRI print(fgc, bgc, char) 
''************************************************
''* Print Character char at current pos          *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    vgabuff[pos++] := char
    vgabuff[pos++] := fgc + (bgc * 16)


PRI printxy(x, y, fgc, bgc, char) 
''************************************************
''* Print Character char at x, y                 *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    getPos(x, y)
    print(fgc, bgc, char)


PRI str(fgc, bgc, string_ptr)
''************************************************
''* Print String at current posv, posc           *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    repeat strsize(string_ptr)
        print(fgc, bgc, byte[string_ptr++])


PRI strxy(x, y, fgc, bgc, string_ptr)
''************************************************
''* Print String at x, y                         *
''* with Foreground-Color fgc and                *
''*      Background-Color bgc                    *
''************************************************

    getPos(x, y)
    str(fgc, bgc, string_ptr)


PRI decr(x, y, fgc, bgc, val) | i, p, npos 

    repeat i from 0 to 3
        num[i] := 0

    pos := 0
    prn(val)
    p := pos-1

    repeat pos
        printxy(x, y, fgc, bgc, num[p]+$30)
        x -= 1
        p--

    if val < 10 
        printxy(x, y, fgc, bgc, $30)


PRI prn(val) | dig

    dig := val // 10
    val := val/10
    if val > 0
        prn(val)
    num[pos++] := dig


DAT

introdat   byte 1, 1, 2, 2, 2, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 2, 1, 1, 1
           byte 1, 2, 1, 1, 1, 2, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 2, 1, 1, 1
           byte 1, 2, 1, 1, 1, 1, 0, 1, 2, 2, 2, 1, 0, 2, 1, 2, 2, 2, 0, 2, 1, 2, 2, 2, 0, 1, 2, 2, 2, 1, 0, 1, 2, 2, 2, 2, 0, 2, 2, 2, 2, 2, 1
           byte 1, 2, 1, 1, 1, 1, 0, 2, 1, 1, 1, 2, 0, 2, 2, 1, 1, 2, 0, 2, 2, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 1, 0, 1, 1, 2, 1, 1, 1
           byte 1, 2, 1, 1, 1, 1, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 2, 2, 2, 2, 0, 2, 1, 1, 1, 1, 0, 1, 1, 2, 1, 1, 1
           byte 1, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 1, 0, 2, 1, 1, 1, 1, 0, 1, 1, 2, 1, 2, 1
           byte 1, 1, 2, 2, 2, 1, 0, 1, 2, 2, 2, 1, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 1, 2, 2, 2, 2, 0, 1, 2, 2, 2, 2, 0, 1, 1, 1, 2, 1, 1

           byte 2, 2, 2, 2, 2, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1
           byte 2, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1
           byte 2, 1, 1, 1, 1, 0, 1, 2, 2, 2, 1, 0, 2, 1, 1, 1, 2, 0, 2, 1, 2, 2, 1
           byte 2, 2, 2, 2, 1, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 2, 1, 1, 2
           byte 2, 1, 1, 1, 1, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 1
           byte 2, 1, 1, 1, 1, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 2, 0, 2, 1, 1, 1, 1
           byte 2, 1, 1, 1, 1, 0, 1, 2, 2, 2, 1, 0, 1, 2, 2, 2, 1, 0, 2, 1, 1, 1, 1


introtext  byte "CONNECT4_42 v1.2 ws 1/2020                            http://propeller.ws-nbg.de", 0


DAT

strHelp    byte "HELP", 0
strPlay1   byte "F1     Player 1", 0
strPlay2   byte "       Player 2", 0
strF2      byte "F2     Auto-Mirror", 0
strF3      byte "F3     Animation", 0
strMove    byte $5D, $5E, "     Move", 0
strDrop1   byte $7F, "      Drop", 0
strDrop2   byte "Space  Drop", 0
strRe      byte "R      Restart Game", 0
strF12     byte "F12    Reboot", 0
strStart   byte "Press ", $22, $53, $22, " to Start", 0
strRestart byte " Press ", $22, $52, $22, " to Restart ", 0
strOn      byte "On ", 0
strOff     byte "Off", 0
strTime    byte "Total Time", 0
strNoKey   byte " Keyboard not found ! ", 0 

strBar     byte $9F, $9F, 0

strLeg     byte $9C, $9F, $9F, $9E, 0

strHat1    byte $01, $02, $03, $04, $05, $06, $07, $08, $09, 0
strHat2    byte $0A, $0B, $0C, $0D, $0E, $0F, $10, $11, $12, 0
strHat3    byte $13, $14, $15, $16, $17, $18, $19, $1A, $1B, 0
strHat4    byte $1C, $1D, $1E, $1F, $3B, $3C, $3D, $3E, $3F, 0



hTab       byte $00, $01, $02, $03
           byte $01, $02, $03, $04
           byte $02, $03, $04, $05
           byte $03, $04, $05, $06

           byte $07, $08, $09, $0A
           byte $08, $09, $0A, $0B
           byte $09, $0A, $0B, $0C
           byte $0A, $0B, $0C, $0D

           byte $0E, $0F, $10, $11
           byte $0F, $10, $11, $12
           byte $10, $11, $12, $13
           byte $11, $12, $13, $14

           byte $15, $16, $17, $18
           byte $16, $17, $18, $19
           byte $17, $18, $19, $1A
           byte $18, $19, $1A, $1B

           byte $1C, $1D, $1E, $1F
           byte $1D, $1E, $1F, $20
           byte $1E, $1F, $20, $21
           byte $1F, $20, $21, $22

           byte $23, $24, $25, $26
           byte $24, $25, $26, $27
           byte $25, $26, $27, $28
           byte $26, $27, $28, $29


vTab       byte $00, $07, $0E, $15
           byte $07, $0E, $15, $1C
           byte $0E, $15, $1C, $23

           byte $01, $08, $0F, $16
           byte $08, $0F, $16, $1D
           byte $0F, $16, $1D, $24

           byte $02, $09, $10, $17
           byte $09, $10, $17, $1E
           byte $10, $17, $1E, $25

           byte $03, $0A, $11, $18
           byte $0A, $11, $18, $1F
           byte $11, $18, $1F, $26

           byte $04, $0B, $12, $19
           byte $0B, $12, $19, $20
           byte $12, $19, $20, $27

           byte $05, $0C, $13, $1A
           byte $0C, $13, $1A, $21
           byte $13, $1A, $21, $28

           byte $06, $0D, $14, $1B
           byte $0D, $14, $1B, $22
           byte $14, $1B, $22, $29


drTab      byte $00, $08, $10, $18
           byte $07, $0F, $17, $1F
           byte $0E, $16, $1E, $26

           byte $01, $09, $11, $19
           byte $08, $10, $18, $20
           byte $0F, $17, $1F, $27

           byte $02, $0A, $12, $1A
           byte $09, $11, $19, $21
           byte $10, $18, $20, $28

           byte $03, $0B, $13, $1B
           byte $0A, $12, $1A, $22
           byte $11, $19, $21, $29


dlTab      byte $06, $0C, $12, $18
           byte $0D, $13, $19, $1F
           byte $14, $1A, $20, $26

           byte $05, $0B, $11, $17
           byte $0C, $12, $18, $1E
           byte $13, $19, $1F, $25

           byte $04, $0A, $10, $16
           byte $0B, $11, $17, $1D
           byte $12, $18, $1E, $24

           byte $03, $09, $0F, $15
           byte $0A, $10, $16, $1C
           byte $11, $17, $1D, $23
           
                                         


tone0      long $000F_070C, $000F_070C, $000E_070C, $000E_070C, $000D_070C, $000D_070C, $000C_070C, $000C_070C
           long $000B_070C, $000B_070C, $000A_070C, $000A_070C, $0009_070C, $0009_070C, $0008_070C, $0000_0000
           long $FFFFFFFF

tone1      long $000F_0300, $000F_02E0, $000F_02C0, $000F_02A0, $000F_0280, $0000_0000
           long $FFFFFFFF


tone2      long $000F_0090, $000F_00B0, $000F_00D0, $000F_00F0
           long $000F_0110, $000F_0130, $000F_0150, $0000_0000
           long $FFFFFFFF


tone3      long $000F_0100, $000F_0200, $000F_0100, $000F_0200, $000F_0100, $0000_0000 
           long $FFFFFFFF


tone4      long $000F_0150, $000F_0130, $000F_0110, $000F_00F0
           long $000F_0100, $000F_00B0, $000F_0090, $0000_0000
           long $FFFFFFFF


tone5      long $000F_0160, $000F_0165, $000F_0160, $000F_0165, $000E_0160, $000E_0165, $000E_0160, $000E_0165 
           long $000D_0160, $000D_0165, $000D_0160, $000D_0165, $000C_0160, $000C_0165, $000C_0160, $000C_0165 
           long $000B_0160, $000B_0165, $000B_0160, $000B_0165, $000A_0160, $000A_0165, $000A_0160, $000A_0165 
           long $0009_0160, $0009_0165, $0009_0160, $0009_0165, $0008_0160, $0008_0165, $0008_0160, $0008_0165 
           long $0007_0160, $0007_0165, $0007_0160, $0007_0165, $0006_0160, $0006_0165, $0006_0160, $0006_0165 
           long $0005_0160, $0005_0165, $0005_0160, $0005_0165, $0004_0160, $0004_0165, $0004_0160, $0004_0165 
           long $0003_0160, $0003_0165, $0003_0160, $0003_0165, $0002_0160, $0002_0165, $0002_0160, $0002_0165 
           long $0001_0160, $0001_0165, $0001_0160, $0001_0165, $0000_0000 
           long $FFFFFFFF
           

tone6      long $000F_02FF, $000F_02FF, $000F_02FF, $000F_02FF, $000F_02FF, $000F_02FF, $000F_02FF, $000F_02FF
           long $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF
           long $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF, $000F_03FF
           long $000F_03FF, $0000_0000
           long $FFFFFFFF


tone7      long $000F_0400, $000F_0400, $0000_0000
           long $FFFFFFFF


noise      long $10_00_03_10
           long $FFFFFFFF

           long $10_00_03_12
           long $FFFFFFFF

           long $10_00_03_14
           long $FFFFFFFF

           long $10_00_03_16
           long $FFFFFFFF

           long $10_00_03_18
           long $FFFFFFFF

           long $10_00_03_1A
           long $FFFFFFFF

           long $10_00_03_1C
           long $FFFFFFFF

           long $10_00_03_1F
           long $FFFFFFFF

           
{{

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}

                       '