''*******************************
''* PLAY.spin v1.1       6/2016 *
''* Author: Werner L. Schneider *
''*******************************
''
''
VAR

    long cog                     ' cog flag/id

    'DO NOT REARRANGE LONGS
    long ay1_regs                ' 14 contiguous longs
    long ay2_regs 

    long buff1A_ptr
    long buff1B_ptr
    long buff2A_ptr
    long buff2B_ptr
    long buff1C_ptr
    long buff2C_ptr

    long flag1A
    long flag1B
    long flag2A
    long flag2B
    long flag1C
    long flag2C


OBJ

    ay1 : "AYcog"
    ay2 : "AYcog"


PUB start(right, left)
    stop

    flag1A := 0
    flag1B := 0
    flag2A := 0
    flag2B := 0
    flag1C := 0
    flag2C := 0

    ay1_Regs := ay1.start(0, left)         ' Start the emulated AY chip in one cog Player 1 left Channel
    ay2_Regs := ay2.start(right, 0)        ' Start the emulated AY chip in one cog Player 2 right Channel 

    ay1.resetRegisters                    ' Reset all AY registers
    ay2.resetRegisters                    ' Reset all AY registers

    ay1.enableTone(true, true, false)   
    ay1.enableNoise(false, false, true) 

    ay2.enableTone(true, true, false)   
    ay2.enableNoise(false, false, true) 

    cog := cognew(@entry, @ay1_regs) + 1


PUB stop
    if cog
        cogstop(cog~ - 1)
    ay1.stop
    ay2.stop
    longfill(@ay1_regs, 0, 14)


PUB ch1A(offs)
 
    buff1A_ptr := offs
    flag1A := 1


PUB ch1A_stop

    ay1.setVolume(0, 0)
    flag1A := 0


PUB ch1Aflag

    return flag1A


PUB ch1B(offs)
 
    buff1B_ptr := offs
    flag1B := 1


PUB ch1B_stop

    ay1.setVolume(1, 0)
    flag1B := 0


PUB ch1Bflag

    return flag1B


PUB ch2A(offs)
 
    buff2A_ptr := offs
    flag2A := 1


PUB ch2A_stop

    ay2.setVolume(0, 0)
    flag2A := 0


PUB ch2Aflag

    return flag2A


PUB ch2B(offs)
 
    buff2B_ptr := offs
    flag2B := 1


PUB ch2B_stop

    ay2.setVolume(1, 0)
    flag2B := 0


PUB ch2Bflag

    return flag2B


PUB ch1C(offs)
 
    buff1C_ptr := offs
    flag1C := 1


PUB ch1C_stop

    ay1.setVolume(2, 0)
    flag1C := 0


PUB ch1Cflag

    return flag1C


PUB ch2C(offs)
 
    buff2C_ptr := offs
    flag2C := 1


PUB ch2C_stop

    ay2.setVolume(2, 0)
    flag2C := 0


PUB ch2Cflag

    return flag2C


DAT

'***********************************
'* Assembly language "play" driver *
'***********************************

                        org     
'
' Entry
'

entry                   movd      :par, #_ay1ptr
                        mov       t1, par
                        mov       t2, #14
:par                    mov       0, t1
                        add       :par, dlsb
                        add       t1, #4
                        djnz      t2, #:par

                        mov       _pd, pdelay
 
'*********************************************************************************************************

                        rdlong    _ay1, _ay1ptr           ' AY #1 Reg 0 Fine Tune Ch A
                        mov       _ay12, _ay1
                        add       _ay12, #2               ' AY #1 Reg 2 Fine Tune Ch B
                        mov       _ay16, _ay12
                        add       _ay16, #2 + 2           ' AY #1 Reg 6 Noise Period
                        mov       _ay1e, _ay16
                        add       _ay1e, #6               ' AY #1 Reg 12 Envelope Ccoarse Tune
                        mov       _ay1s, _ay1e
                        add       _ay1s, #1               ' AY #1 Reg 13 Shape Control

                        mov       _ay1volA, _ay1
                        add       _ay1volA, #8            ' AY #1 Reg 8 Volume Ch A
                        mov       _ay1volB, _ay1volA
                        add       _ay1volB, #1            ' AY #1 Reg 9 Volume Ch B
                        mov       _ay1volC, _ay1volB
                        add       _ay1volC, #1            ' AY #1 Reg 10 Volume Ch C

'*********************************************************************************************************

                        rdlong    _ay2, _ay2ptr           ' AY #2 Reg 0 Fine Tune Ch A
                        mov       _ay22, _ay2
                        add       _ay22, #2               ' AY #2 Reg 2 Fine Tune Ch B
                        mov       _ay26, _ay22
                        add       _ay26, #2 + 2           ' AY #2 Reg 6 Noise Period
                        mov       _ay2e, _ay26
                        add       _ay2e, #6               ' AY #2 Reg 12 Envelope Ccoarse Tune
                        mov       _ay2s, _ay2e
                        add       _ay2s, #1               ' AY #2 Reg 13 Shape Control

                        mov       _ay2volA, _ay2
                        add       _ay2volA, #8            ' AY #2 Reg 8 Volume Ch A
                        mov       _ay2volB, _ay2volA
                        add       _ay2volB, #1            ' AY #2 Reg 9 Volume Ch B
                        mov       _ay2volC, _ay2volB
                        add       _ay2volC, #1            ' AY #2 Reg 10 Volume Ch C

'*********************************************************************************************************

bigloop                 mov       _delay, cnt          
                        add       _delay, _pd

'*********************************************************************************************************

entry1                  rdlong    t1, _flagptr1A          ' Read Ch 1A Flag
                        cmp       t1, #0            wz
             if_z       jmp       #entry2     
                        cmp       t1, #1            wz 
             if_z       add       t1, #1
             if_z       wrlong    t1, _flagptr1A
             if_z       rdlong    _buff1A, _buffptr1A
                        rdlong    _tone, _buff1A
                        add       _buff1A, #4
                        mov       _vol, _tone
                        shr       _vol, #16
                        and       _vol, #$F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready1A
                        and       _tone, tonemask

                        wrword    _tone, _ay1             ' AY #1 Reg 0/1 Ch A Fine Tune/Coarse Tune
                        wrbyte    _vol, _ay1volA          ' AY #1 Reg 8 Volume Ch A

'*********************************************************************************************************

entry2                  rdlong    t2, _flagptr1B          ' Read Ch 1B Flag
                        cmp       t2, #0            wz
             if_z       jmp       #entry3     
                        cmp       t2, #1            wz 
             if_z       add       t2, #1
             if_z       wrlong    t2, _flagptr1B
             if_z       rdlong    _buff1B, _buffptr1B
                        rdlong    _tone, _buff1B  
                        add       _buff1B, #4
                        mov       _vol, _tone
                        shr       _vol, #16
                        and       _vol, #$F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready1B
                        and       _tone, tonemask

                        wrword    _tone, _ay12            ' AY #1 Reg 2/3 Ch B Fine Tune/Coarse Tune
                        wrbyte    _vol, _ay1volB          ' AY #1 Reg 9 Volume Ch B

'*********************************************************************************************************

entry3                  rdlong    t3, _flagptr2A          ' Read Ch 2A Flag
                        cmp       t3, #0            wz
             if_z       jmp       #entry4     
                        cmp       t3, #1            wz 
             if_z       add       t3, #1
             if_z       wrlong    t3, _flagptr2A
             if_z       rdlong    _buff2A, _buffptr2A
                        rdlong    _tone, _buff2A
                        add       _buff2A, #4
                        mov       _vol, _tone
                        shr       _vol, #16
                        and       _vol, #$F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready2A
                        and       _tone, tonemask

                        wrword    _tone, _ay2             ' AY #2 Reg 0/1 Ch A Fine Tune/Coarse Tune
                        wrbyte    _vol, _ay2volA          ' AY #2 Reg 8 Volume Ch A

'*********************************************************************************************************

entry4                  rdlong    t4, _flagptr2B          ' Read Ch 2B Flag
                        cmp       t4, #0            wz
             if_z       jmp       #entry5     
                        cmp       t4, #1            wz 
             if_z       add       t4, #1
             if_z       wrlong    t4, _flagptr2B
             if_z       rdlong    _buff2B, _buffptr2B
                        rdlong    _tone, _buff2B  
                        add       _buff2B, #4
                        mov       _vol, _tone
                        shr       _vol, #16
                        and       _vol, #$F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready2B
                        and       _tone, tonemask

                        wrword    _tone, _ay22            ' AY #2 Reg 2/3 Ch B Fine Tune/Coarse Tune
                        wrbyte    _vol, _ay2volB          ' AY #2 Reg 9 Volume Ch B

'*********************************************************************************************************

entry5                  rdlong    t5, _flagptr1C          ' Read Ch 1C Flag
                        cmp       t5, #0            wz
             if_z       jmp       #entry6     
                        cmp       t5, #1            wz 
             if_z       add       t5, #1
             if_z       wrlong    t5, _flagptr1C
             if_z       rdlong    _buff1C, _buffptr1C
                        rdlong    _tone, _buff1C        
                        add       _buff1C, #4
                        mov       _env, _tone
                        shr       _env, #8
                        and       _env, #$FF
                        mov       _shape, _tone
                        shr       _shape, #16
                        and       _shape, #$0F
                        mov       _vol, _tone
                        shr       _vol, #24
                        and       _vol, #$1F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready1C
                        and       _tone, noisemask

                        wrbyte    _tone, _ay16            ' AY #1 Reg 6 Noise Period
                        wrbyte    _env, _ay1e             ' AY #1 Reg 12 Envelope Coarse Tune
                        wrbyte    _shape, _ay1s           ' AY #1 Reg 13 Shape Control
                        wrbyte    _vol, _ay1volC          ' AY #1 Reg 10 Volume Ch C

'*********************************************************************************************************

entry6                  rdlong    t6, _flagptr2C          ' Read Ch C2 Flag
                        cmp       t6, #0            wz
             if_z       jmp       #entry7     
                        cmp       t6, #1            wz 
             if_z       add       t6, #1
             if_z       wrlong    t6, _flagptr2C
             if_z       rdlong    _buff2C, _buffptr2C
                        rdlong    _tone, _buff2C  
                        add       _buff2C, #4
                        mov       _env, _tone
                        shr       _env, #8
                        and       _env, #$FF
                        mov       _shape, _tone
                        shr       _shape, #16
                        and       _shape, #$0F
                        mov       _vol, _tone
                        shr       _vol, #24
                        and       _vol, #$1F
                        cmp       _tone, endmask    wz
             if_z       jmp       #ready2C
                        and       _tone, noisemask

                        wrbyte    _tone, _ay26            ' AY #2 Reg 6 Noise Period
                        wrbyte    _env, _ay2e             ' AY #2 Reg 12 Envelope Coarse Tune
                        wrbyte    _shape, _ay2s           ' AY #2 Reg 13 Shape Control
                        wrbyte    _vol, _ay2volC          ' AY #2 Reg 10 Volume Ch C

'*********************************************************************************************************

entry7                  cmp       t1, #0            wz
             if_z       cmp       t2, #0            wz
             if_z       cmp       t3, #0            wz
             if_z       cmp       t4, #0            wz
             if_z       cmp       t5, #0            wz
             if_z       cmp       t6, #0            wz
             if_nz      waitcnt   _delay, _pd        
                        jmp       #bigloop

'*********************************************************************************************************

ready1A                 mov       t1, #0
                        wrlong    t1, _flagptr1A
                        wrbyte    t1, _ay1volA             ' AY #1 Reg 8 Volume Ch A
                        jmp       #entry2

'*********************************************************************************************************

ready1B                 mov       t2, #0
                        wrlong    t2, _flagptr1B
                        wrbyte    t2, _ay1volB             ' AY #1 Reg 9 Volume Ch B
                        jmp       #entry3

'*********************************************************************************************************

ready2A                 mov       t3, #0
                        wrlong    t3, _flagptr2A
                        wrbyte    t3, _ay2volA             ' AY #2 Reg 8 Volume Ch A
                        jmp       #entry4

'*********************************************************************************************************

ready2B                 mov       t4, #0
                        wrlong    t4, _flagptr2B
                        wrbyte    t4, _ay2volB             ' AY #1 Reg 9 Volume Ch B
                        jmp       #entry5

'*********************************************************************************************************

ready1C                 mov       t5, #0
                        wrlong    t5, _flagptr1C
'                        wrbyte    t3, _ayvol3             ' AY-Reg 10 Volume Ch C
                        jmp       #entry6

'*********************************************************************************************************

ready2C                 mov       t6, #0
                        wrlong    t6, _flagptr2C
'                        wrbyte    t3, _ayvol3             ' AY-Reg 10 Volume Ch C
                        jmp       #entry7

'*********************************************************************************************************



                        ' Data

dlsb                    long      1 << 9

pdelay                  long      80_000_000 / 50

endmask                 long      $FFFFFFFF
tonemask                long      $00000FFF
noisemask               long      $0000001F

'
' Uninitialized data
'
t1                      res       1
t2                      res       1
t3                      res       1
t4                      res       1
t5                      res       1
t6                      res       1

_delay                  res       1
_pd                     res       1
_ay                     res       1

_ay1ptr                 res       1               ' 14 params
_ay2ptr                 res       1
_buffptr1A              res       1
_buffptr1B              res       1
_buffptr2A              res       1
_buffptr2B              res       1
_buffptr1C              res       1
_buffptr2C              res       1
_flagptr1A              res       1
_flagptr1B              res       1
_flagptr2A              res       1
_flagptr2B              res       1
_flagptr1C              res       1
_flagptr2C              res       1

_ay1                    res       1
_ay12                   res       1
_ay16                   res       1
_ay1e                   res       1
_ay1s                   res       1
_ay1volA                res       1
_ay1volB                res       1
_ay1volC                res       1

_ay2                    res       1
_ay22                   res       1
_ay26                   res       1
_ay2e                   res       1
_ay2s                   res       1
_ay2volA                res       1
_ay2volB                res       1
_ay2volC                res       1

_buff1A                 res       1
_buff1B                 res       1
_buff2A                 res       1
_buff2B                 res       1
_buff1C                 res       1
_buff2C                 res       1

_tone                   res       1
_vol                    res       1
_env                    res       1
_shape                  res       1

                        fit     $1f0

{{

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}